@@ -1,4 +1,4 @@
-# $Id: test.pl,v 1.10 2002/06/05 10:47:50 cosimo Exp $
+# $Id: test.pl,v 1.12 2002/06/17 20:11:19 Cosimo Exp $
 #
 # Before `make install' is performed this script should be runnable with
 # `make test'. After `make install' it should work as `perl test.pl'
@@ -8,7 +8,7 @@
 # Change 1..1 below to 1..last_test_to_print .
 # (It may become useful if the test is moved to ./t subdirectory.)
 
-BEGIN { $| = 1; print "1..6\n"; }
+BEGIN { $| = 1; print "1..12\n"; }
 END {print "not ok 1\n" unless $loaded;}
 use lib '.';
 use Modem;
@@ -68,7 +68,7 @@ if( $Device::Modem::port eq 'NONE' || $Device::Modem::port eq '' ) {
 
 NOTICE
 
-	print "skip $_\n" for (2..6);
+	print "skip $_\n" for (2..12);
 
 	exit;
 
@@ -195,6 +195,42 @@ if( $modem->is_active() ) {
 }
 
 
+# --- 11 ---
+print 'testing S registers read/write...', "\n";
+
+my $reg = 1;
+my $v1 = $modem->S_register($reg);
+my $v2 = $modem->S_register($reg, 72);
+my $v3 = $modem->S_register($reg);
+my $v4 = $modem->S_register($reg, $v1);
+my $v5 = $modem->S_register($reg);
+
+if( $v1 eq $v5 && $v1 == $v5 &&
+    $v2 == 72  && $v3 == 72  &&
+    $v4 eq $v1 && $v4 == $v1 ) {
+	print "ok 11\n";
+} else {
+	$not_connected_guess++;
+	print "not ok 11\n";
+}
+
+print 'testing status of modem signals...', "\n";
+
+my $signals_on = 0;
+my %status = $modem->status();
+foreach( keys %status ) {
+	print "$_ signal is ", $status{$_} ? 'on' : 'off', "\n";
+	$signals_on++ if $status{$_};
+}
+
+if( $signals_on > 1 ) {
+	print "ok 12\n";
+} else {
+	print "not ok 12\n";
+	$not_connected_guess++;
+}
+
+
 if( $not_connected_guess >= 4 ) {
 
 
